/**
 * Copyright (C) 2008 Ovea <dev@testatoo.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.testatoo.cartridge.core.component;

import org.junit.BeforeClass;
import org.junit.Test;
import org.testatoo.cartridge.WebTest;
import org.testatoo.core.ComponentException;
import org.testatoo.core.component.DropDown;

import java.util.Arrays;
import java.util.List;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;
import static org.junit.Assert.*;
import static org.testatoo.core.ComponentFactory.*;

public class DropDownTest extends WebTest {

    @BeforeClass
    public static void setUp() {
        page().open("Select.html");
    }

    @Test
    public void can_find_dropdown_by_id() {
        dropdown("elements");

        try {
            dropdown("otherDropdown");
            fail();
        } catch (ComponentException e) {
            assertThat(e.getMessage(), is("Cannot find component defined by id=otherDropdown"));
        }
    }

    @Test
    public void exception_thrown_if_component_not_a_dropdown() {
        try {
            dropdown("cities");
            fail();
        } catch (ComponentException e) {
            assertThat(e.getMessage(), is("The component with id=cities is not a DropDown but a ListBox"));
        }
    }

    @Test
    public void test_list_values() {
        // List without explicit values (in this case, the value is set with the content)
        DropDown countriesList = dropdown("countries");
        assertThat(countriesList.values(), hasItems("Canada", "France", "Spain"));

        // List with explicit values
        DropDown elementsList = dropdown("elements");
        assertThat(elementsList.values(), hasItems("1", "5", "84", "20", "88"));
        assertThat(elementsList.values(), hasItems("20", "5", "84", "1", "88"));

        List<String> values = dropdown("elements").values();
        assertThat(values.containsAll(Arrays.asList("1", "5", "84", "20", "88")), is(true));
    }

    @Test
    public void test_can_select_value() {
        DropDown dropdown = dropdown("elements");

        assertThat(dropdown.selectedValue(), is(""));
        dropdown.select("84");
        assertThat(dropdown.selectedValue(), is("84"));
    }

    // Test specific for Html
    @Test
    public void can_select_item() {
        // List without explicit values (in this case, the value is set with the content)
        DropDown countriesList = dropdown("countries");
        countriesList.select("France");
        assertThat(countriesList.selectedValue(), is("France"));
        countriesList.select("Spain");
        assertThat(countriesList.selectedValue(), is("Spain"));

        // List with explicit values
        DropDown elementsList = dropdown("elements");
        elementsList.select("84");
        assertThat(elementsList.selectedValue(), is("84"));
        elementsList.select("20");
        assertThat(elementsList.selectedValue(), is("20"));
    }

    @Test
    public void test_label() {
        assertThat(dropdown("elements").label(), is("Elements list"));
    }

    @Test
    public void test_toString() {
        assertThat(dropdown("elements").toString(), is("class org.testatoo.cartridge.html4.element.DropDown with state : enabled:true, visible:true, values:[1, 5, 84, 20, 88], selectedValues:[], label:Elements list"));
    }
}
