/**
 * Copyright (C) 2008 Ovea <dev@testatoo.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.testatoo.cartridge.html4;

import org.junit.Before;
import org.junit.Test;
import org.testatoo.core.ComponentException;
import org.testatoo.core.component.Component;

import java.util.List;

import static org.junit.Assert.*;
import static org.mockito.Mockito.*;

public class ByTest {

    private HtmlEvaluator evaluator;
    private String id = "myId";

    @Before
    public void setUp() {
        evaluator = mock(HtmlEvaluator.class);
    }

    @Test
    public void try_to_find_component_by_id_fail_if_pass_null() {
        try {
            By.id(null);
            fail();
        } catch (IllegalArgumentException e) {
            assertEquals(e.getMessage(), "Cannot find component with a null id.");
        }
    }

    @Test
    public void can_find_component_by_id() {
        when(evaluator.existComponent(id)).thenReturn(true);

        By by = By.id(id);

        Component component = by.findComponent(evaluator);
        assertEquals(component.id(), id);
    }

    @Test
    public void can_find_components_by_id() {
        when(evaluator.existComponent(id)).thenReturn(true);

        By by = By.id(id);

        List<Component> components = by.findComponents(evaluator);
        assertEquals(components.size(), 1);
        assertEquals(components.get(0).id(), id);
    }

    @Test
    public void can_find_component_by_id_fail_if_component_is_not_available() {
        when(evaluator.existComponent(id)).thenReturn(false);

        By by = By.id(id);
        try {
            by.findComponent(evaluator);
            fail();
        } catch (ComponentException e) {
            assertEquals(e.getMessage(), "Cannot find component defined by id=myId");
        }
    }

    // ----------------------------------------------------------------------------------------------
    @Test
    public void try_to_find_component_by_xpath_fail_if_pass_null() {
        try {
            By.xpath(null);
            fail();
        } catch (IllegalArgumentException e) {
            assertEquals(e.getMessage(), "Cannot find component when xpath is null.");
        }
    }

    @Test
    public void can_find_component_by_xpath() {
        when(evaluator.existComponent(id)).thenReturn(true);
        when(evaluator.elementId("xpath://button[@name='test']")).thenReturn(id);

        By by = By.xpath("//button[@name='test']");

        Component component = by.findComponent(evaluator);
        assertEquals(component.id(), id);
    }

    @Test
    public void can_find_components_by_xpath() {
        when(evaluator.existComponent("myId_1")).thenReturn(true);
        when(evaluator.existComponent("myId_2")).thenReturn(true);
        when(evaluator.existComponent("myId_3")).thenReturn(true);

        String[] result = {"myId_1", "myId_2", "myId_3"};
        when(evaluator.elementsId("xpath://button[@name='test']")).thenReturn(result);

        By by = By.xpath("//button[@name='test']");

        List<Component> components = by.findComponents(evaluator);
        assertEquals(components.size(), 3);
    }

    // ----------------------------------------------------------------------------------------------
    @Test
    public void try_to_find_component_by_jquery_fail_if_pass_null() {
        try {
            By.jQuery(null);
            fail();
        } catch (IllegalArgumentException e) {
            assertEquals(e.getMessage(), "Cannot find component when jQueryExpression is null.");
        }
    }

    @Test
    public void can_find_component_by_jquery() {
        when(evaluator.existComponent(id)).thenReturn(true);
        when(evaluator.elementId("jquery:div#content .photo")).thenReturn(id);

        By by = By.jQuery("div#content .photo");

        Component component = by.findComponent(evaluator);
        assertEquals(component.id(), id);
    }

    @Test
    public void can_find_components_by_jquery() {
        when(evaluator.existComponent("myId_1")).thenReturn(true);
        when(evaluator.existComponent("myId_2")).thenReturn(true);
        when(evaluator.existComponent("myId_3")).thenReturn(true);

        String[] result = {"myId_1", "myId_2", "myId_3"};

        when(evaluator.elementsId("jquery:#tableId tr")).thenReturn(result);

        By by = By.jQuery("#tableId tr");

        List<Component> components = by.findComponents(evaluator);
        assertEquals(components.size(), 3);
    }
}
