/**
 * Copyright (C) 2008 Ovea <dev@testatoo.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.testatoo.cartridge.html4.element;

import org.junit.BeforeClass;
import org.junit.Test;
import org.testatoo.cartridge.WebTest;
import org.testatoo.cartridge.html4.By;
import org.testatoo.core.ComponentException;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;
import static org.junit.Assert.*;
import static org.testatoo.cartridge.html4.HtmlComponentFactory.*;

public class ButtonTest extends WebTest {

    @BeforeClass
    public static void setUp() {
        page().open("Button.html");
    }

    @Test
    public void exception_thrown_if_component_not_a_html_button() {
        try {
            findButton(By.id("country"));
            fail();
        } catch (ComponentException e) {
            assertThat(e.getMessage(), is("The component with id=country is not a Button but a DropDown"));
        }
    }

    @Test
    public void exception_thrown_if_component_is_a_radio_button() {
        try {
            findButton(By.id("radio"));
            fail();
        } catch (ComponentException e) {
            assertThat(e.getMessage(), is("The component with id=radio is not a Button but a Radio"));
        }
    }

    @Test
    public void can_get_button_type() {
        assertThat(button("submitImage_1").type(), is(InputType.image));
        assertThat(button("submit_1").type(), is(InputType.submit));
        assertThat(button("reset_1").type(), is(InputType.reset));
        assertThat(button("submitImage_1").type(), is(InputType.image));
        assertThat(button("button").type(), is(InputType.submit));
    }

    @Test
    public void can_get_the_icon() {
        assertThat(button("submitImage_1").icon(), containsString("images/forward.png"));
        assertThat(button("submit_1").icon(), is(""));
        assertThat(button("button").icon(), containsString("images/forward.png"));
    }

    @Test
    public void test_i18nAttributes() {
        Button button = button("button_2");
        assertThat(button.direction(), is(Direction.righttoleft));
        assertThat(button.language(), is("fr"));
    }

    @Test
    public void test_coreAttributes() {
        Button button = button("button_2");

        assertThat(button.id(), is("button_2"));
        assertThat(button.classname(), is("myClass"));
        assertThat(button.style(), containsString("color:black"));
        assertThat(button.title(), is("buttonTitle"));
    }

    @Test
    public void test_specifics_attributes() {
        Button button = button("button");
        assertThat(button.name(), is("myButtonName"));
        assertThat(button.value(), is("myButtonValue"));
        assertThat(button.alt(), is(""));
        assertThat(button.tabindex(), is(0));
        assertThat(button.accesskey(), is(""));
        assertThat(button.type(), is(InputType.submit));

        Button button2 = button("button_2");
        assertThat(button2.name(), is("myButton"));
        assertThat(button2.value(), is("buttonValue"));
        assertThat(button2.alt(), is("short description"));
        assertThat(button2.tabindex(), is(5));
        assertThat(button2.accesskey(), is("C"));
    }

    @Test
    public void test_enability() {
        Button button1 = button("button");
        assertThat(button1.isEnabled(), is(true));
        assertThat(button1.isDisabled(), is(false));

        Button disabledButton = button("disabledButton");
        assertThat(disabledButton.isEnabled(), is(false));
        assertThat(disabledButton.isDisabled(), is(true));
    }

    @Test
    public void can_find_button_by_name() {
        findButton(By.name("buttonName"));
        findButton(By.name("myButtonName"));
        try {
            findButton(By.name("otherButtonName"));
            fail();
        } catch (ComponentException e) {
            assertThat(e.getMessage(), is("Cannot find component defined by name=otherButtonName"));
        }
    }

    @Test
    public void can_find_button_by_title() {
        findButton(By.title("buttonTitle"));

        try {
            findButton(By.title("otherButtonTitle"));
            fail();
        } catch (ComponentException e) {
            assertThat(e.getMessage(), is("Cannot find component defined by title=otherButtonTitle"));
        }
    }

    @Test
    public void test_toString() {
        assertThat(button("button").toString(), is("class org.testatoo.cartridge.html4.element.Button with state : enabled:true, visible:true, text:MyButtonText, icon:images/forward.png"));
        assertThat(button("button_1").toString(), is("class org.testatoo.cartridge.html4.element.Button with state : enabled:true, visible:true, text:buttonValue, icon:"));
    }
}
