/**
 * Copyright (C) 2008 Ovea <dev@testatoo.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.testatoo.cartridge.html4.element;

import org.junit.Before;
import org.junit.Test;
import org.testatoo.cartridge.WebTest;
import org.testatoo.cartridge.html4.By;
import org.testatoo.core.ComponentException;
import org.testatoo.core.input.Mouse;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;
import static org.junit.Assert.*;
import static org.testatoo.cartridge.html4.HtmlComponentFactory.*;
import static org.testatoo.core.Language.*;

public class FormTest extends WebTest {

    @Before
    public void setUp() {
        page().open("Form.html");
    }

    @Test
    public void can_find_form_by_id() {
        findForm(By.id("myForm"));

        try {
            findForm(By.id("otherForm"));
            fail();
        } catch (ComponentException e) {
            assertThat(e.getMessage(), is("Cannot find component defined by id=otherForm"));
        }
    }

    @Test
    public void exception_thrown_if_component_not_a_html_form() {
        try {
            findForm(By.id("email"));
            fail();
        } catch (ComponentException e) {
            assertThat(e.getMessage(), is("The component with id=email is not a Form but a InputText"));
        }
    }

    @Test
    public void test_i18nAttributes() {
        Form myForm = findForm(By.id("myForm"));

        assertThat(myForm.direction(), is(Direction.lefttoright));
        assertThat(myForm.language(), is("fr"));
    }

    @Test
    public void test_coreAttributes() {
        Form myForm = findForm(By.id("myForm"));

        assertThat(myForm.id(), is("myForm"));
        assertThat(myForm.classname(), is("myClass"));
        assertThat(myForm.style(), containsString("color:black"));
        assertThat(myForm.title(), is("formTitle"));
    }

    @Test
    public void test_specifics_attributes() {
        Form myForm = findForm(By.id("myForm"));

        assertThat(myForm.method(), is(Method.post));
        assertThat(myForm.action(), is("Exit.html"));
        assertThat(myForm.enctype(), is("application/x-www-form-urlencoded"));
        assertThat(myForm.accept(), is("all"));
        assertThat(myForm.name(), is("formName"));
        // Not supported by IE
//        assertThat(myForm.acceptCharset(), is("UTF-8"));
    }

    @Test
    public void if_attribute_are_not_defined_then_return_default_values() {
        Form myForm3 = findForm(By.id("myForm3"));

        assertThat(myForm3.method(), is(Method.get));
        assertThat(myForm3.enctype(), is("application/x-www-form-urlencoded"));
        assertThat(myForm3.accept(), is(""));
        assertThat(myForm3.name(), is(""));
        assertThat(myForm3.acceptCharset(), is("UNKNOWN"));
    }

    @Test
    public void test_contains() {
        Form myForm = findForm(By.id("myForm"));

        Select citiesList = select("cities");
        assertThat(myForm.contains(citiesList), is(true));

        InputText inputText = inputText("lastname");
        assertThat(myForm.contains(inputText), is(true));

        Radio radio = radio("male");
        assertThat(myForm.contains(radio), is(true));

        CheckBox checkBox = findCheckBox(By.name("yes"));
        assertThat(myForm.contains(checkBox), is(true));

        Button button = button("submitImage");
        assertThat(myForm.contains(button), is(true));
    }

    @Test
    public void can_reset_a_form() {
        type("Joe", on(inputText("firstname")));
        type("Blow", into(inputText("lastname")));
        enter("email@noname.com", into(inputText("email")));

        check(radio("male"));
        check(findCheckBox(By.name("yes")));

        on(select("cities")).select("Casablanca");

        form("myForm").reset();

        assertThat(inputText("firstname").value(), is(""));
        assertThat(inputText("lastname").value(), is(""));
        assertThat(inputText("email").value(), is(""));

        assertThat(radio("male").isChecked(), is(false));
        assertThat(findCheckBox(By.name("yes")).isChecked(), is(false));

        assertThat(select("cities").selectedOptions().get(0).content(), is("New York"));

        type("Joe", on(inputText("firstname")));
        type("Blow", into(inputText("lastname")));
        enter("email@noname.com", into(inputText("email")));

        check(radio("male"));
        check(findCheckBox(By.name("yes")));

        on(select("cities")).select("Casablanca");

        on(form("myForm")).reset();

        assertThat(inputText("firstname").value(), is(""));
        assertThat(inputText("lastname").value(), is(""));
        assertThat(inputText("email").value(), is(""));

        assertThat(radio("male").isChecked(), is(false));
        assertThat(findCheckBox(By.name("yes")).isChecked(), is(false));

        assertThat(select("cities").selectedOptions().get(0).content(), is("New York"));
    }

    @Test
    public void can_submit_a_form() {
        assertThat(page().title(), is("Form tests"));

        Button submitButton = button("submitImage");
        clickOn(submitButton);

        assertThat(page().title(), is("Exit page"));

        page().open("Form.html");
        assertThat(page().title(), is("Form tests"));
        Mouse.clickOn(submitButton);

        assertThat(page().title(), is("Exit page"));

        page().open("Form.html");
        assertThat(page().title(), is("Form tests"));

        form("myForm").submit();

        assertThat(page().title(), is("Exit page"));
    }

    @Test
    public void can_find_form_by_name() {
        findForm(By.name("formName"));

        try {
            findForm(By.name("otherFormName"));
            fail();
        } catch (ComponentException e) {
            assertThat(e.getMessage(), is("Cannot find component defined by name=otherFormName"));
        }
    }

    @Test
    public void can_find_form_by_title() {
        findForm(By.title("formTitle"));

        try {
            findForm(By.title("otherFormTitle"));
            fail();
        } catch (ComponentException e) {
            assertThat(e.getMessage(), is("Cannot find component defined by title=otherFormTitle"));
        }
    }

    @Test
    public void test_toString() {
        assertThat(findForm(By.id("myForm")).toString(), is("class org.testatoo.cartridge.html4.element.Form with state : enabled:true, visible:true, action:Exit.html"));
    }
}
