/**
 * Copyright (C) 2008 Ovea <dev@testatoo.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.testatoo.cartridge.html4.element;

import org.junit.BeforeClass;
import org.junit.Test;
import org.testatoo.cartridge.WebTest;
import org.testatoo.cartridge.html4.By;
import org.testatoo.core.ComponentException;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;
import static org.junit.Assert.*;
import static org.testatoo.cartridge.html4.HtmlComponentFactory.*;

public class InputTextTest extends WebTest {

    @BeforeClass
    public static void setUp() {
        page().open("InputText.html");
    }

    @Test
    public void can_find_inputText_by_id() {
        findInputText(By.id("input_1"));

        try {
            findInputText(By.id("input_2"));
            fail();
        } catch (ComponentException e) {
            assertThat(e.getMessage(), is("Cannot find component defined by id=input_2"));
        }

        try {
            findInputText(By.id("inputTextError"));
            fail();
        } catch (ComponentException e) {
            assertThat(e.getMessage(), is("The component with id=inputTextError is not a TextField but a CheckBox"));
        }
    }

    @Test
    public void test_i18nAttributes() {
        InputText myText = findInputText(By.id("text"));

        assertThat(myText.direction(), is(Direction.righttoleft));
        assertThat(myText.language(), is("fr"));
    }

    @Test
    public void test_coreAttributes() {
        InputText myText = findInputText(By.id("text"));

        assertThat(myText.id(), is("text"));
        assertThat(myText.classname(), is("myClass"));
        assertThat(myText.style(), containsString("color:black"));
        assertThat(myText.title(), is("textTitle"));
    }

    @Test
    public void test_specifics_attributes() {
        InputText myText = findInputText(By.id("text"));

        assertThat(myText.name(), is("myText"));
        assertThat(myText.value(), is("textValue"));
        assertThat(myText.alt(), is("short description"));
        assertThat(myText.tabindex(), is(5));
        assertThat(myText.accesskey(), is("C"));
        assertThat(myText.type(), is(InputType.text));
        assertThat(myText.toString(), is("class org.testatoo.cartridge.html4.element.InputText with state : enabled:false, visible:true, value:textValue, label:Text label, maxLength:20"));

        assertThat(findInputText(By.id("input_1")).size(), is(20));

        InputText myTextDef = findInputText(By.id("text_def"));

        assertThat(myTextDef.name(), is(""));
        assertThat(myTextDef.value(), is(""));
        assertThat(myTextDef.alt(), is(""));
        assertThat(myTextDef.tabindex(), is(0));
        assertThat(myTextDef.accesskey(), is(""));
        assertThat(myTextDef.type(), is(InputType.text));
        assertThat(myTextDef.size(), is(10));
    }

    @Test
    public void test_enability() {
        assertThat(findInputText(By.id("text")).isReadOnly(), is(true));
        assertThat(findInputText(By.id("readonlytext")).isReadOnly(), is(true));
        assertThat(findInputText(By.id("input_language")).isReadOnly(), is(false));

        assertThat(findInputText(By.id("disabledtext")).isEnabled(), is(false));
        assertThat(findInputText(By.id("input_language")).isEnabled(), is(true));
    }

    @Test
    public void can_test_max_length() {
        assertThat(findInputText(By.id("input_1")).maxLength(), is(10));
        assertThat(findInputText(By.id("input_language")).maxLength(), is(20));
    }

    @Test
    public void test_label() {
        assertThat(findInputText(By.id("text")).label(), is("Text label"));
        assertThat(findInputText(By.name("cellnumber")).label(), is("Cell phone number"));
        assertThat(findInputText(By.name("homenumber")).label(), is("Home phone number"));
    }

    @Test
    public void can_find_inputText_by_name() {
        findInputText(By.name("inputTextName"));

        try {
            findInputText(By.name("otherInputTextName"));
            fail();
        } catch (ComponentException e) {
            assertThat(e.getMessage(), is("Cannot find component defined by name=otherInputTextName"));
        }
    }

    @Test
    public void can_find_inputText_by_title() {
        findInputText(By.title("inputTextTitle"));

        try {
            findInputText(By.title("otherInputTextTitle"));
            fail();
        } catch (ComponentException e) {
            assertThat(e.getMessage(), is("Cannot find component defined by title=otherInputTextTitle"));
        }
    }

    @Test
    public void test_toString() {
        assertThat(findInputText(By.id("text")).toString(),
                is("class org.testatoo.cartridge.html4.element.InputText with state : enabled:false, visible:true, value:textValue, label:Text label, maxLength:20"));
    }
}
