/**
 * Copyright (C) 2008 Ovea <dev@testatoo.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.testatoo.cartridge.html4.element;

import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.testatoo.cartridge.WebTest;
import org.testatoo.cartridge.html4.By;
import org.testatoo.core.ComponentException;
import org.testatoo.core.input.Mouse;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;
import static org.junit.Assert.*;
import static org.testatoo.cartridge.html4.HtmlComponentFactory.*;

public class LinkTest extends WebTest {

    @Before
    public void setUp() {
        page().open("Link.html");
    }

    @Test
    public void can_find_a_by_id() {
        findA(By.id("a_1"));

        try {
            findA(By.id("a_0"));
            fail();
        } catch (ComponentException e) {
            assertThat(e.getMessage(), is("Cannot find component defined by id=a_0"));
        }
    }

    @Test
    public void exception_thrown_if_component_not_a_html_a() {
        try {
            findA(By.id("radio"));
            fail();
        } catch (ComponentException e) {
            assertThat(e.getMessage(), is("The component with id=radio is not a Link but a Radio"));
        }
    }

    @Test
    public void test_a_i18nAttributes() {
        A a = findA(By.id("a_1"));

        assertThat(a.direction(), is(Direction.lefttoright));
        assertThat(a.language(), is("fr"));
    }

    @Test
    public void test_a_coreAttributes() {
        A a = findA(By.id("a_1"));

        assertThat(a.id(), is("a_1"));
        assertThat(a.classname(), is("myAClass"));
        assertThat(a.style(), containsString("color:black"));
        assertThat(a.title(), is("aTitle"));
    }

    @Test
    public void test_a_specifics_attributes() {
        A a = findA(By.id("a_1"));

        assertThat(a.type(), is("text/html"));
        assertThat(a.name(), is("aName"));
        assertThat(a.charset(), is("utf-8"));

        assertThat(a.href(), is("Exit.html"));
        assertThat(a.hreflang(), is("fr"));

        assertThat(a.rel(), is("foo"));
        assertThat(a.rev(), is("bar"));
        assertThat(a.accesskey(), is("C"));
        assertThat(a.shape(), is(Shape.rect));
        assertThat(a.coords(), is("rect:10,10,10,10"));
        assertThat(a.tabindex(), is(1));
    }

    @Test
    public void can_find_a_by_name() {
        findA(By.name("aName"));

        try {
            findA(By.name("otheraName"));
            fail();
        } catch (ComponentException e) {
            assertThat(e.getMessage(), is("Cannot find component defined by name=otheraName"));
        }
    }

    @Test
    public void can_find_a_by_title() {
        findA(By.title("aTitle"));

        try {
            findA(By.title("otheraTitle"));
            fail();
        } catch (ComponentException e) {
            assertThat(e.getMessage(), is("Cannot find component defined by title=otheraTitle"));
        }
    }

    @Test
    public void click_on_A_change_page() {
        A a_link = findA(By.id("a_1"));
        assertThat(page().title(), is("Link tests"));
        Mouse.clickOn(a_link);
        assertThat(page().title(), is("Exit page"));
    }

    @Test
    public void testA_toString() {
        assertThat(findA(By.id("a_1")).toString(), is("class org.testatoo.cartridge.html4.element.A with state : enabled:true, visible:true, text:Link 1 - a type, reference:Exit.html"));
    }

    //------------------------------------------
    @Test
    public void can_find_link_by_id() {
        findLink(By.id("link_1"));

        try {
            findLink(By.id("link_0"));
            fail();
        } catch (ComponentException e) {
            assertThat(e.getMessage(), is("Cannot find component defined by id=link_0"));
        }
    }

    @Test
    public void exception_thrown_if_component_not_a_html_link() {
        try {
            findLink(By.id("radio"));
            fail();
        } catch (ComponentException e) {
            assertThat(e.getMessage(), is("The component with id=radio is not a Link but a Radio"));
        }
    }

    @Test
    public void test_link_i18nAttributes() {
        Link link = link("link_1");

        assertThat(link.direction(), is(Direction.righttoleft));
        assertThat(link.language(), is("en"));
    }

    @Test
    public void test_link_coreAttributes() {
        Link link = link("link_1");

        assertThat(link.id(), is("link_1"));
        assertThat(link.classname(), is("myLINKClass"));
        assertThat(link.style(), containsString("color:orange"));
        assertThat(link.title(), is("linkTitle"));
    }

    @Test
    public void test_link_specifics_attributes() {
        Link link = link("link_1");

        assertThat(link.type(), is("text/html"));
        assertThat(link.charset(), is("utf-8"));

        assertThat(link.href(), is("Exit.html"));
        assertThat(link.hreflang(), is("fr"));

        assertThat(link.rel(), is("next, index"));
        assertThat(link.rev(), is("prev, chapter"));
    }

    @Test
    public void test_enability() {
        Link link = link("link_1");
        assertThat(link.isVisible(), is(false));
    }

    @Test
    public void testLink_toString() {
        Link link = link("link_1");
        assertThat(link.toString(), is("class org.testatoo.cartridge.html4.element.Link with state : enabled:true, visible:false, reference:Exit.html"));
    }
}
